#include <stdint.h>
#include <sys/types.h>
#include <sys/vfs.h>
#include <dirent.h>
#include <errno.h>
#include "ff.h"

struct vfs *
_vfs_open_sdsdcard(unsigned cdiv, int pclk, int pcmd, int pdat0, int ppow, int pled)
{
    unsigned r;
    struct vfs *v;
    struct __using("filesys/sdfatfs/fatfs.cc") *FFS;
    FATFS *FatFs;
    unsigned long long pmask;

    FFS = _gc_alloc_managed(sizeof(*FFS));
    FatFs = _gc_alloc_managed(sizeof(*FatFs));

#ifdef _DEBUG
    __builtin_printf("open sdcard: using pins: %d %d %d %d %d\n", pclk, pcmd, pdat0, ppow, pled);
#endif    
    pmask = (1ULL << pclk) | (1ULL << pcmd) | (15ULL << pdat0) | (1ULL << ppow) | (1ULL << pled);
    if (!_usepins(pmask)) {
        _seterror(EBUSY);
        return 0;
    }
    FFS->f_pinmask = pmask;
    r = FFS->disk_setpins(0, cdiv, pclk, pcmd, pdat0, ppow, pled);
    if (r == 0) {
        r = FFS->f_mount(FatFs, "", 0);
    }
    if (r != 0) {
#ifdef _DEBUG
       __builtin_printf("sd card fs_init failed: result=[%d]\n", r);
       _waitms(1000);
#endif
       _freepins(pmask);
       _seterror(-r);
       return 0;
    }
    v = FFS->get_vfs(FFS);
#ifdef _DEBUG
    {
        unsigned *ptr = (unsigned *)v;
        __builtin_printf("sd card get_vfs: returning %x\n", (unsigned)ptr);
    }
#endif
    return v;
}
